from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    Text,
    Boolean,
)
from datetime import datetime

from ...database import Base


class UserSavedSearchModel(Base):
    __tablename__ = 'user_saved_searches'

    id = Column(Integer, primary_key=True)
    user_id = Column(String(128), nullable=False)
    search_name = Column(String(128), nullable=False)
    search_query = Column(Text, nullable=False)
    search_type = Column(String(128), nullable=False)
    is_deleted = Column(Boolean, nullable=True)
    created_at = Column(DateTime, nullable=False)
    last_processed = Column(DateTime, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Boolean,
    Text,
)
from datetime import datetime

from ...database import Base


class ScientificLinkModel(Base):
    __tablename__ = 'scientific_links'

    id = Column(Integer, primary_key=True)
    doi = Column(String(191), nullable=False)
    pubmed_id = Column(
        Integer,
        ForeignKey('pubmed.id'),
        nullable=True,
    )
    date = Column(DateTime, nullable=True)
    title = Column(String(255), nullable=True)
    type = Column(String(50), nullable=True)
    abstract = Column(Text, nullable=True)
    journal_title = Column(String(255), nullable=True)
    inbound_ref_count = Column(Integer, nullable=True)
    outbound_ref_count = Column(Integer, nullable=True)
    orig_file_url = Column(String(255), nullable=True)
    best_oa_location = Column(String(255), nullable=True)
    method = Column(String(128), nullable=True)
    verification = Column(String(128), nullable=True)
    is_deleted = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)
from ...utils.utils import pre_load_date_fields


class ScientificLinkResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    doi = fields.String(required=True, validate=not_blank)
    pubmed_id = fields.Integer(allow_none=True)
    date = fields.DateTime(allow_none=True)
    title = fields.String(allow_none=True)
    type = fields.String(allow_none=True)
    abstract = fields.String(allow_none=True)
    journal_title = fields.String(allow_none=True)
    inbound_ref_count = fields.Integer(allow_none=True)
    outbound_ref_count = fields.Integer(allow_none=True)
    orig_file_url = fields.String(allow_none=True)
    best_oa_location = fields.String(allow_none=True)
    method = fields.String(allow_none=True)
    verification = fields.String(allow_none=True)
    is_deleted = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%dT%H%M%S',
        )
        return in_data



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    Text,
    Float,
)
from datetime import datetime

from ...database import Base


class NotificationModel(Base):
    __tablename__ = 'notifications'

    id = Column(Integer, primary_key=True)
    saved_search_id = Column(
        Integer,
        ForeignKey('user_saved_searches.id'),
        nullable=False,
    )
    source_type = Column(String(50), nullable=False)
    source_table = Column(String(50), nullable=False)
    artifact_id = Column(String(128), nullable=False)
    text = Column(Text, nullable=True)
    score = Column(Float, nullable=True)
    source_id = Column(Integer, nullable=False)
    source_detail = Column(Text, nullable=False)
    seen_at = Column(DateTime, nullable=True)
    processed_at = Column(DateTime, nullable=True)
    date = Column(DateTime, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class MergerOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    announcement_date = fields.DateTime(required=True)
    news_id = fields.Integer(required=True)
    deal_value = fields.String(allow_none=True)
    price = fields.String(allow_none=True)
    advisors = fields.String(allow_none=True)
    counsels = fields.String(allow_none=True)
    sentences = fields.String(allow_none=True)
    reviewed = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime()




from marshmallow import (
    Schema,
    fields,
)


class UserSavedSearchResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    user_id = fields.String(required=True)
    search_name = fields.String(required=True)
    search_query = fields.String(required=True)
    search_type = fields.String(required=True)
    is_deleted = fields.Boolean(allow_none=True)
    created_at = fields.DateTime(required=True)
    last_processed = fields.DateTime(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    String,
)

from ...database import Base


class ViewPublicationConditionNormModel(Base):
    __tablename__ = "_view_publication_condition_norm"

    id = Column(Integer, primary_key=True)
    condition_norm_cui = Column(String(128), nullable=False, index=True)
    date = Column(DateTime, nullable=False)
    table_name = Column(String(50), nullable=False)
    table_id = Column(Integer, nullable=False)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    pre_load,
    validate,
)
from ...utils.utils import pre_load_date_fields


class IPOOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    ipo_date = fields.DateTime(required=True)
    deal_value = fields.Decimal(allow_none=True)
    price = fields.String(allow_none=True)
    ticker = fields.String(allow_none=True)
    type = fields.String(allow_none=True)
    news_id = fields.Integer()
    currency = fields.String(allow_none=True)
    company_sec_id = fields.Integer(allow_none=True)
    company_ous_id = fields.Integer(allow_none=True)
    underwriters = fields.String(allow_none=True)
    counsels = fields.String(allow_none=True)
    reviewed = fields.String(allow_none=True)
    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data, **kwargs):
        date_fields = ['ipo_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%dT%H%M%S',
        )
        return in_data



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    ForeignKey,
)

from ...database import Base


class NCTSubsidiaryModel(Base):
    __tablename__ = "nct_subsidiaries"

    id = Column(Integer, primary_key=True)
    nct_id = Column(String(128), nullable=False, index=True)
    nct_study_id = Column(
        Integer,
        ForeignKey('nct_study.id'),
        nullable=False,
    )
    subsidiary_id = Column(
        Integer,
        ForeignKey('subsidiaries.id'),
        nullable=False,
    )
    lead_or_collaborator = Column(String(50))
    study_start_date = Column(DateTime, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint("nct_study_id", "subsidiary_id"),)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class NotificationResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    saved_search_id = fields.Integer(required=True)
    source_type = fields.String(required=True)
    source_table = fields.String(required=True)
    source_id = fields.Integer(required=True)
    artifact_id = fields.String(required=True)
    text = fields.String(allow_none=True)
    score = fields.Float(allow_none=True)
    source_detail = fields.String(required=True)
    seen_at = fields.DateTime(allow_none=True)
    processed_at = fields.DateTime(allow_none=True)
    date = fields.DateTime(required=True)
    updated_at = fields.DateTime()



from sqlalchemy import (
    Integer,
    Column,
    Boolean,
    DateTime,
    ForeignKey
)
from datetime import datetime
from ...database import Base


class NCTCompanySECFilingModel(Base):
    __tablename__ = "nct_companies_sec_filings"

    id = Column(Integer, primary_key=True)
    nct_study_id = Column(
        Integer,
        ForeignKey('nct_study.id'),
        nullable=False,
    )
    sec_filing_id = Column(
        Integer,
        ForeignKey('companies_sec_filings.id'),
        nullable=False,
    )
    sec_filing_date = Column(DateTime, nullable=True)
    is_deleted = Column(
        Boolean,
        nullable=True
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from marshmallow import (
    Schema,
    fields,
    validate,
)


class NCTCompanyFilingResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    nct_study_id = fields.Integer(required=True)
    company_filing_id = fields.Integer(required=True)
    is_deleted = fields.Boolean(allow_none=True)
    date = fields.DateTime(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)
